# cgVertexExample.tcl
#
# Original files from: http://developer.nvidia.com/Cg
# This is the example called runtime_ogl as included in the Cg Toolkit.
#
# Modified for Tcl3D by Paul Obermeier 2005/11/07
# See www.tcl3d.org for the Tcl3D extension.

package require Tk
package require tcl3d 0.3.2

if { [info procs tcl3dHaveCg] ne "" } {
    if { ![tcl3dHaveCg] } {
        tk_messageBox -icon error -type ok -title "Error" \
                      -message "You do not have Cg installed."
        exit
    }
}

set LightDiffuse {1.0 0.0 0.0 1.0}  
set LightPosition {1.0 1.0 1.0 0.0}

# Show errors occuring in the Togl callbacks.
proc bgerror { msg } {
    tk_messageBox -icon error -type ok -message "Error: $msg"
    exit
}

# Print info message into widget a the bottom of the window.
proc PrintInfo { msg } {
    if { [winfo exists .fr.info] } {
        .fr.info configure -text $msg
    }
}

proc drawBoxTcl { size type } {
    set n {
        {-1.0  0.0  0.0}
        { 0.0  1.0  0.0}
        { 1.0  0.0  0.0}
        { 0.0 -1.0  0.0}
        { 0.0  0.0  1.0}
        { 0.0  0.0 -1.0}
    }
    set faces {
        {0 1 2 3}
        {3 2 6 7}
        {7 6 5 4}
        {4 5 1 0}
        {5 6 2 1}
        {7 4 0 3}
    }

    set sm [expr -0.5 * $size]
    set sp [expr  0.5 * $size]

    set v [list \
        [list $sm $sm $sm] \
        [list $sm $sm $sp] \
        [list $sm $sp $sp] \
        [list $sm $sp $sm] \
        [list $sp $sm $sm] \
        [list $sp $sm $sp] \
        [list $sp $sp $sp] \
        [list $sp $sp $sm] \
        ]

    for { set i 0 } { $i < 6 } { incr i } {
        set f [lindex $faces $i]
        glBegin $type
        glNormal3fv [lindex $n $i]
 
        cgGLSetParameter3f $::TestColorParam 1.0 0.0 0.0
        glVertex3fv [lindex $v [lindex $f 0]]

        cgGLSetParameter3f $::TestColorParam 0.0 1.0 0.0
        glVertex3fv [lindex $v [lindex $f 1]]

        cgGLSetParameter3f $::TestColorParam 0.0 0.0 1.0
        glVertex3fv [lindex $v [lindex $f 2]]

        cgGLSetParameter3f $::TestColorParam 1.0 1.0 1.0
        glVertex3fv [lindex $v [lindex $f 3]]
        glEnd
    }
}

proc DrawCube {} {
    cgGLBindProgram $::Program
    set retVal [tcl3dCgGetError $::Context]
    if { $retVal ne "" } {
        error "cgGLBindProgram DrawCube: $retVal"
    }

    if { [info exists ::KdParam] } {
        cgGLSetParameter4f $::KdParam 1.0 1.0 0.0 1.0
    } 
    if { [info exists ::ModelViewProjParam] } {
        cgGLSetStateMatrixParameter $::ModelViewProjParam \
                                CG_GL_MODELVIEW_PROJECTION_MATRIX \
                                CG_GL_MATRIX_IDENTITY
    }

    cgGLEnableProfile $::profile

    drawBoxTcl 2 $::GL_QUADS

    cgGLDisableProfile $::profile
}

proc tclDisplayFunc { toglwin } {
    update
    glClear [expr $::GL_COLOR_BUFFER_BIT | $::GL_DEPTH_BUFFER_BIT]
    DrawCube
    $toglwin swapbuffers
}

proc tclCreateFunc { toglwin } {
    glLightfv GL_LIGHT0 GL_DIFFUSE $::LightDiffuse
    glLightfv GL_LIGHT0 GL_POSITION $::LightPosition
    glEnable GL_LIGHT0
    glDisable GL_LIGHTING

    glEnable GL_DEPTH_TEST

    glMatrixMode GL_PROJECTION
    gluPerspective 40.0 1.0 1.0 10.0
    glMatrixMode GL_MODELVIEW
    gluLookAt 0.0 0.0 5.0 \
            0.0 0.0 0.0 \
            0.0 1.0 0.0

    glTranslatef 0.0 0.0 -1.0
    glRotatef 60 1.0 0.0 0.0
    glRotatef -20 0.0 0.0 1.0

    InitCgShader
}

proc InitCgShader {} {
    set ::profile [tcl3dCgFindProfile $::CG_PROFILE_VP20 $::CG_PROFILE_ARBVP1]
    if { $::profile == "" } {
        exit 1 
    }

    set ::Context [cgCreateContext]
    set retVal [tcl3dCgGetError $::Context]
    if { $retVal ne "" } {
        error "cgCreateContext: $retVal"
    }

    set cgFile [tcl3dGetExtFile [file join [file dirname [info script]] "cgGL_vertex_example.cg"]]
    set ::Program [cgCreateProgramFromFile $::Context \
                                        CG_SOURCE $cgFile \
                                        $::profile "main" "NULL"]
    set retVal [tcl3dCgGetError $::Context]
    if { $retVal ne "" } {
        error "cgCreateProgramFromFile: $retVal"
    }
    tcl3dCgPrintProgramInfo $::Program "cgGL_vertex_example.cg"

    if { [info exists ::Program] } {
        cgGLLoadProgram $::Program
        set retVal [tcl3dCgGetError $::Context]
        if { $retVal ne "" } {
            error "cgGLLoadProgram: $retVal"
        }

        set ::KdParam [cgGetNamedParameter $::Program "Kd"]

        set ::ModelViewProjParam [cgGetNamedParameter $::Program "ModelViewProj"]

        set ::TestColorParam [cgGetNamedParameter $::Program "IN.TestColor"]
    }
}

proc CreateWindow {} {
    frame .fr
    pack .fr -expand 1 -fill both
    togl .fr.toglwin -width 400 -height 400 \
                     -double true -depth true \
                     -createproc tclCreateFunc \
                     -displayproc tclDisplayFunc
    label .fr.info

    grid .fr.toglwin -row 0 -column 0 -sticky news
    grid .fr.info    -row 1 -column 0 -sticky news
    grid rowconfigure .fr 0 -weight 1
    grid columnconfigure .fr 0 -weight 1

    wm title . "Tcl3D demo: Simple Cg vertex shader"

    bind . <Key-Escape> "exit"
}


CreateWindow

PrintInfo [format "Running on %s with a %s (OpenGL %s, Tcl %s)" \
           $tcl_platform(os) [glGetString GL_RENDERER] \
           [glGetString GL_VERSION] [info patchlevel]]
