# toglInCanvas.tcl
#
# Tcl3D demo using a Togl window and some button widgets inserted into a canvas.
#
# Author: Paul Obermeier
# Date: 2006-12-08

package require tcl3d

# Font to be used in the Tk listbox.
set listFont {-family {Courier} -size 10}

# Rotation of the box
set rotAngle 0.0

# Offset of the Togl window.
set woff 0
set hoff 0

# Show errors occuring in the Togl callbacks.
proc bgerror { msg } {
    tk_messageBox -icon error -type ok -message "Error: $msg"
    exit
}

# Print info message into widget at the bottom of the window.
proc PrintInfo { msg } {
    if { [winfo exists .fr.info] } {
        .fr.info configure -text $msg
    }
}

proc ShiftImg { dir } {
    global myCanv woff hoff

    switch $dir {
        "SE" { set woff [expr $woff + 10]
               set hoff [expr $hoff + 10] 
        }
        "NE" { set woff [expr $woff + 10]
               set hoff [expr $hoff - 10] 
        }
        "SW" { set woff [expr $woff - 10]
               set hoff [expr $hoff + 10] 
        }
        "NW" { set woff [expr $woff - 10]
               set hoff [expr $hoff - 10] 
        }
    }
    ReshapeCanvas $myCanv
}

proc tclCreateFunc { toglwin } {
    glClearColor 0.5 0.5 0.5 0.5
    glShadeModel GL_SMOOTH
    glClearDepth 1.0
    glEnable GL_DEPTH_TEST
}

proc tclDisplayFunc { toglwin } {
    glClear [expr $::GL_COLOR_BUFFER_BIT | $::GL_DEPTH_BUFFER_BIT] 

    glLoadIdentity
    glTranslatef 0.0 0.0 -7.0
    glRotatef $::rotAngle 1.0 1.0 1.0

    glBegin GL_QUADS
        glColor3f 0.0 1.0 0.0
        glVertex3f  1.0  1.0 -1.0
        glVertex3f -1.0  1.0 -1.0
        glVertex3f -1.0  1.0  1.0
        glVertex3f  1.0  1.0  1.0
        glColor3f 1.0 0.5 0.0  
        glVertex3f  1.0 -1.0  1.0
        glVertex3f -1.0 -1.0  1.0
        glVertex3f -1.0 -1.0 -1.0
        glVertex3f  1.0 -1.0 -1.0
        glColor3f 1.0 0.0 0.0   
        glVertex3f  1.0  1.0  1.0
        glVertex3f -1.0  1.0  1.0
        glVertex3f -1.0 -1.0  1.0
        glVertex3f  1.0 -1.0  1.0
        glColor3f 1.0 1.0 0.0   
        glVertex3f  1.0 -1.0 -1.0
        glVertex3f -1.0 -1.0 -1.0
        glVertex3f -1.0  1.0 -1.0
        glVertex3f  1.0  1.0 -1.0
        glColor3f 0.0 0.0 1.0    
        glVertex3f -1.0  1.0  1.0
        glVertex3f -1.0  1.0 -1.0
        glVertex3f -1.0 -1.0 -1.0
        glVertex3f -1.0 -1.0  1.0
        glColor3f 1.0 0.0 1.0    
        glVertex3f  1.0  1.0 -1.0
        glVertex3f  1.0  1.0  1.0
        glVertex3f  1.0 -1.0  1.0
        glVertex3f  1.0 -1.0 -1.0
    glEnd
    set ::rotAngle [expr $::rotAngle -0.5]
    
    $toglwin swapbuffers
}

proc tclReshapeFunc { toglwin  w h } {
    if { $h == 0 } {
        set h 1 
    }
    glViewport 0 0 $w $h
    glMatrixMode GL_PROJECTION
    glLoadIdentity

    gluPerspective 45.0 [expr double($w)/double($h)] 0.1 100.0

    glMatrixMode GL_MODELVIEW
    glLoadIdentity
}

proc ReshapeCanvas { c } {
    set w [winfo width $c]
    set h [winfo height $c]
    set cw [expr $w / 2]
    set ch [expr $h / 2]
    $c coords TOGLWIN [expr $cw + $::woff] [expr $ch + $::hoff]
    $c itemconfigure TOGLWIN -width $w -height $h
    $c coords SE $w $h
    $c coords SW 0  $h 
    $c coords NE $w 0 
    $c coords NW 0  0
    $c coords TEXT1 $cw 20
    $c coords TEXT2 $cw [expr $h -20]
}

proc Animate {} {
    global myCanv

    $myCanv.toglwin postredisplay
    set ::animateId [after idle Animate]
}

proc StartAnimation {} {
    if { ! [info exists ::animateId] } {
        Animate
    }
}

proc StopAnimation {} {
    if { [info exists ::animateId] } {
        after cancel $::animateId 
        unset ::animateId
    }
}

# Put all exit related code here.
proc ExitProg {} {
    exit
}

proc CreateWindow {} {
    global myCanv

    set w 300
    set h 300
    set cw [expr $w / 2]
    set ch [expr $h / 2]

    # Create top frame.
    frame .fr
    pack .fr -expand 1 -fill both

    # Add a canvas and info widgets.
    set myCanv .fr.c
    canvas  $myCanv -bg green
    listbox .fr.usage -height 3 -font $::listFont
    label   .fr.info
    grid $myCanv   -row 0 -column 0 -sticky news
    grid .fr.usage -row 1 -column 0 -sticky news
    grid .fr.info  -row 2 -column 0 -sticky news
    grid rowconfigure    .fr 0 -weight 1
    grid columnconfigure .fr 0 -weight 1
    wm title . "Tcl3D demo: Togl window in canvas"

    # Fill the canvas with a togl window and some button widgets.
    togl $myCanv.toglwin -width $cw -height $ch \
                        -double true -depth true \
                        -createproc tclCreateFunc \
                        -displayproc tclDisplayFunc \
                        -reshapeproc tclReshapeFunc
    $myCanv create window $cw $ch -window $myCanv.toglwin \
            -anchor center -tags "TOGLWIN"

    button $myCanv.b_se -relief groove -text "SE" -command {ShiftImg "SE"} \
           -repeatdelay 200 -repeatinterval 100
    $myCanv create window $w $h -window $myCanv.b_se -anchor se -tags "SE"

    button $myCanv.b_sw -relief groove -text "SW" -command {ShiftImg "SW"} \
           -repeatdelay 200 -repeatinterval 100
    $myCanv create window 0 $h -window $myCanv.b_sw -anchor sw -tags "SW"

    button $myCanv.b_ne -relief groove -text "NE" -command {ShiftImg "NE"} \
           -repeatdelay 200 -repeatinterval 100
    $myCanv create window $w 0 -window $myCanv.b_ne -anchor ne -tags "NE"

    button $myCanv.b_nw -relief groove -text "NW" -command {ShiftImg "NW"} \
           -repeatdelay 200 -repeatinterval 100
    $myCanv create window 0 0 -window $myCanv.b_nw -anchor nw -tags "NW"

    # Create some canvas text items for printing messages.
    $myCanv create text $cw 20 -tags "TEXT1" -anchor center \
            -text "This is the canvas background"
    $myCanv create text $cw [expr $h -20]  -tags "TEXT2" -anchor center \
            -text "This is the canvas background"

    bind $myCanv <Configure> "ReshapeCanvas .fr.c"

    wm protocol . WM_DELETE_WINDOW "ExitProg"
    bind . <Key-Escape> "ExitProg"

    bind $myCanv.toglwin <1> "StartAnimation"
    bind $myCanv.toglwin <2> "StopAnimation"

    .fr.usage insert end "Key-Escape Exit"
    .fr.usage insert end "Mouse-1|2  Start|Stop animation"
    .fr.usage insert end "Button     Move Togl window"
    .fr.usage configure -state disabled
}

CreateWindow

PrintInfo [format "Running on %s with a %s (OpenGL %s, Tcl %s)" \
           $tcl_platform(os) [glGetString GL_RENDERER] \
           [glGetString GL_VERSION] [info patchlevel]]
if { [file tail [info script]] == [file tail $::argv0] } {
    # If started directly from tclsh or wish, then start animation.
    update
    StartAnimation
}
