# double.tcl
#
# An example of the OpenGL red book modified to work with Tcl3D.
# The original C sources are Copyright (c) 1993-2003, Silicon Graphics, Inc.
# The Tcl3D sources are Copyright (c) 2005, Paul Obermeier.
# See file LICENSE for complete license information.
#
# This is a simple double buffered program.
# Pressing the left mouse button rotates the rectangle.
# Pressing the middle mouse button stops the rotation.

package require tcl3d

set spin 0.0

# Font to be used in the Tk listbox.
set listFont {-family {Courier} -size 10}

# Show errors occuring in the Togl callbacks.
proc bgerror { msg } {
    tk_messageBox -icon error -type ok -message "Error: $msg"
    exit
}

# Print info message into widget a the bottom of the window.
proc PrintInfo { msg } {
    if { [winfo exists .fr.info] } {
        .fr.info configure -text $msg
    }
}

proc tclDisplayFunc { toglwin } {
    glClear GL_COLOR_BUFFER_BIT
    glPushMatrix
    glRotatef $::spin 0.0 0.0 1.0
    glColor3f 1.0 1.0 1.0
    glRectf -25.0 -25.0 25.0 25.0
    glPopMatrix
    glFlush
 
    $toglwin swapbuffers
}

proc StartAnimation {} {
    global spin

    set spin [expr $spin + 2.0]
    if { $spin > 360.0 } {
        set spin [expr $spin - 360.0]
    }
    .fr.toglwin postredisplay
    set ::spinId [after idle StartAnimation]
}

proc StopAnimation {} {
    if { [info exists ::spinId] } {
        after cancel $::spinId 
    }
}

proc tclCreateFunc { toglwin } {
    glClearColor 0.0 0.0 0.0 0.0
    glShadeModel GL_FLAT
}

proc tclReshapeFunc { toglwin  w h } {
    glViewport 0 0 $w $h
    glMatrixMode GL_PROJECTION
    glLoadIdentity
    glOrtho -50.0 50.0 -50.0 50.0 -1.0 1.0
    glMatrixMode GL_MODELVIEW
    glLoadIdentity
}

frame .fr
pack .fr -expand 1 -fill both
togl .fr.toglwin -width 250 -height 250 -double true \
                 -createproc tclCreateFunc \
                 -reshapeproc tclReshapeFunc \
                 -displayproc tclDisplayFunc 
listbox .fr.usage -font $::listFont -height 3
label   .fr.info
grid .fr.toglwin -row 0 -column 0 -sticky news
grid .fr.usage   -row 1 -column 0 -sticky news
grid .fr.info    -row 2 -column 0 -sticky news
grid rowconfigure .fr 0 -weight 1
grid columnconfigure .fr 0 -weight 1
wm title . "Tcl3D demo: OpenGL Red Book example double"

bind . <Key-Escape> "exit"
bind .fr.toglwin <1> "StartAnimation"
bind .fr.toglwin <2> "StopAnimation" 

.fr.usage insert end "Mouse-1    Start animation"
.fr.usage insert end "Mouse-2    Stop animation"
.fr.usage insert end "Key-Escape Exit"
.fr.usage configure -state disabled

PrintInfo [format "Running on %s with a %s (OpenGL %s, Tcl %s)" \
           $tcl_platform(os) [glGetString GL_RENDERER] \
           [glGetString GL_VERSION] [info patchlevel]]
